package com.ningdatech.pmapi.performance.controller;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import com.ningdatech.pmapi.performance.manage.PerformanceAppraisalManage;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalCreateDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalEditDTO;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * @Classname PerformanceAppraisalController
 * @Description
 * @Date 2023/6/19 14:16
 * @Author PoffyZhang
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "绩效评价控制器")
@RequestMapping("/api/v1/performance-appraisal")
public class PerformanceAppraisalController {

    private final PerformanceAppraisalManage performanceAppraisalManage;

    @GetMapping("/list")
    @ApiOperation("绩效列表")
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        return performanceAppraisalManage.list(req);
    }

    @GetMapping("/detail/{id}")
    @ApiOperation("绩效详情")
    public PerformanceAppraisalVO detail(@PathVariable Long id) {
        return performanceAppraisalManage.detail(id);
    }

    @PostMapping("/create")
    @ApiOperation("新建绩效")
    @WebLog("新建绩效")
    public String create(PerformanceAppraisalCreateDTO createDTO) {
        return performanceAppraisalManage.create(createDTO);
    }

    @PostMapping("/edit")
    @ApiOperation("编辑绩效")
    @WebLog("编辑绩效")
    public String edit(PerformanceAppraisalEditDTO editDTO) {
        return performanceAppraisalManage.edit(editDTO);
    }
}
