package com.ningdatech.pmapi.performance.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.performance.convert.PerformanceAppraisalConveter;
import com.ningdatech.pmapi.performance.model.dto.AppraisalProjectDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalCreateDTO;
import com.ningdatech.pmapi.performance.model.dto.PerformanceAppraisalEditDTO;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisal;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalApplication;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalProject;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalVO;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalApplicationService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalProjectService;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalService;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname PerformanceAppraisalManage
 * @Description
 * @Date 2023/6/19 14:18
 * @Author PoffyZhang
 */
@Component
@Slf4j
@AllArgsConstructor
public class PerformanceAppraisalManage {

    private final IPerformanceAppraisalService performanceAppraisalService;

    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;

    private final IPerformanceAppraisalApplicationService performanceAppraisalApplicationService;

    private final IProjectService projectService;

    private final IProjectApplicationService applicationService;

    /**
     * 绩效列表
     * @param req
     * @return
     */
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        Page<PerformanceAppraisal> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisal> wrapper = Wrappers.lambdaQuery(PerformanceAppraisal.class)
                        .like(StringUtils.isNotBlank(req.getName()),PerformanceAppraisal::getName,req.getName())
                        .ge(Objects.nonNull(req.getStart()),PerformanceAppraisal::getCreateOn,req.getStart())
                        .le(Objects.nonNull(req.getEnd()),PerformanceAppraisal::getCreateOn,req.getEnd())
                        .orderByDesc(PerformanceAppraisal::getUpdateOn);
        performanceAppraisalService.page(page,wrapper);
        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<Long> paIds  = page.getRecords().stream().map(PerformanceAppraisal::getId).collect(Collectors.toList());
        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .in(PerformanceAppraisalProject::getAppraisalId, paIds));

        Map<String,Project> projectMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalProject>> papsGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paps)){
            papsGroupMap = paps.stream().collect(Collectors.groupingBy(PerformanceAppraisalProject::getAppraisalId));

            List<String> projectCodes = paps.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toList());
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getProjectCode, projectCodes)
                    .eq(Project::getNewest, Boolean.TRUE));
            if(CollUtil.isNotEmpty(projects)){
                projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
            }
        }

        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .in(PerformanceAppraisalApplication::getAppraisalId, paIds));
        Map<Long,ProjectApplication> applicationMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalApplication>> paasGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paas)){
            paasGroupMap = paas.stream().collect(Collectors.groupingBy(PerformanceAppraisalApplication::getAppraisalId));

            List<Long> applicationIds = paas.stream().map(PerformanceAppraisalApplication::getApplicationId).collect(Collectors.toList());
            List<ProjectApplication> apps = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .in(ProjectApplication::getId, applicationIds));
            if(CollUtil.isNotEmpty(apps)){
                applicationMap = apps.stream().collect(Collectors.toMap(ProjectApplication::getId,a -> a));
            }
        }

        Map<Long, List<PerformanceAppraisalProject>> finalPapsGroupMap = papsGroupMap;
        Map<Long,List<PerformanceAppraisalApplication>> finalPaasGroupMap = paasGroupMap;
        Map<String,Project> finalProjectMap = projectMap;
        Map<Long,ProjectApplication> finalApplicationMap = applicationMap;
        List<PerformanceAppraisalVO> res = page.getRecords().stream().map(p -> {
            PerformanceAppraisalVO vo = BeanUtil.copyProperties(p,PerformanceAppraisalVO.class);
            if(finalPapsGroupMap.containsKey(p.getId())){
                vo.setProjects(PerformanceAppraisalConveter.convertProjects(finalPapsGroupMap.get(p.getId()),finalProjectMap));
            }
            if(finalPaasGroupMap.containsKey(p.getId())){
                vo.setApplications(PerformanceAppraisalConveter.convertApplications(finalPaasGroupMap.get(p.getId()),finalApplicationMap));
            }
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }

    public PerformanceAppraisalVO detail(Long id) {

        PerformanceAppraisal pa = performanceAppraisalService.getById(id);
        VUtils.isTrue(Objects.isNull(pa)).throwMessage("该计划不存在");

        PerformanceAppraisalVO vo = BeanUtil.copyProperties(pa,PerformanceAppraisalVO.class);
        //绩效关联的项目
        List<PerformanceAppraisalProject> paps = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId, id));
        Map<String,Project> projectMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalProject>> papsGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paps)){
            papsGroupMap = paps.stream().collect(Collectors.groupingBy(PerformanceAppraisalProject::getAppraisalId));

            List<String> projectCodes = paps.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toList());
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .in(Project::getProjectCode, projectCodes)
                    .eq(Project::getNewest, Boolean.TRUE));
            if(CollUtil.isNotEmpty(projects)){
                projectMap = projects.stream().collect(Collectors.toMap(Project::getProjectCode,p -> p));
            }
        }

        //绩效关联的应用
        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId, id));
        Map<Long,ProjectApplication> applicationMap = Maps.newHashMap();
        Map<Long,List<PerformanceAppraisalApplication>> paasGroupMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(paas)){
            paasGroupMap = paas.stream().collect(Collectors.groupingBy(PerformanceAppraisalApplication::getAppraisalId));

            List<Long> applicationIds = paas.stream().map(PerformanceAppraisalApplication::getApplicationId).collect(Collectors.toList());
            List<ProjectApplication> apps = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .in(ProjectApplication::getId, applicationIds));
            if(CollUtil.isNotEmpty(apps)){
                applicationMap = apps.stream().collect(Collectors.toMap(ProjectApplication::getId,a -> a));
            }
        }

        Map<Long, List<PerformanceAppraisalProject>> finalPapsGroupMap = papsGroupMap;
        Map<Long,List<PerformanceAppraisalApplication>> finalPaasGroupMap = paasGroupMap;
        Map<String,Project> finalProjectMap = projectMap;
        Map<Long,ProjectApplication> finalApplicationMap = applicationMap;
        if(finalPapsGroupMap.containsKey(vo.getId())){
            vo.setProjects(PerformanceAppraisalConveter.convertProjects(finalPapsGroupMap.get(vo.getId()),finalProjectMap));
        }
        if(finalPaasGroupMap.containsKey(vo.getId())){
            vo.setApplications(PerformanceAppraisalConveter.convertApplications(finalPaasGroupMap.get(vo.getId()),finalApplicationMap));
        }
        return vo;
    }

    /**
     * 新建计划
     * @param createDTO
     * @return
     */
    public String create(PerformanceAppraisalCreateDTO createDTO) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        deduplicationName(createDTO);

        PerformanceAppraisal pa = BeanUtil.copyProperties(createDTO,PerformanceAppraisal.class);
        pa.setCreateOn(LocalDateTime.now());
        pa.setCreateBy(user.getUsername());
        pa.setUpdateOn(LocalDateTime.now());
        pa.setUpdateBy(user.getUsername());
        performanceAppraisalService.save(pa);

        if(CollUtil.isNotEmpty(createDTO.getProjects())){
            saveProjects(createDTO,pa.getId(),user);
        }

        if(CollUtil.isNotEmpty(createDTO.getApplicationIds())){
            saveApplications(createDTO,pa.getId(),user);
        }

        return "新建成功";
    }


    /**
     * 编辑计划
     * @param editDTO
     * @return
     */
    public String edit(PerformanceAppraisalEditDTO editDTO) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        deduplicationName(editDTO);

        PerformanceAppraisal pa = BeanUtil.copyProperties(editDTO,PerformanceAppraisal.class);

        pa.setUpdateOn(LocalDateTime.now());
        pa.setUpdateBy(user.getUsername());
        performanceAppraisalService.updateById(pa);

        if(CollUtil.isNotEmpty(editDTO.getProjects())){
            saveProjects(editDTO,user);
        }

        if(CollUtil.isNotEmpty(editDTO.getApplicationIds())){
            saveApplications(editDTO,user);
        }

        return "编辑成功";
    }

    /**
     * 名称去重
     * @param dto
     */
    private void deduplicationName(PerformanceAppraisalCreateDTO dto) {
        long count = performanceAppraisalService.count(Wrappers.lambdaQuery(PerformanceAppraisal.class)
                .eq(PerformanceAppraisal::getName, dto.getName()));
        VUtils.isTrue(count > 0).throwMessage("名称已经存在");
    }

    private void deduplicationName(PerformanceAppraisalEditDTO dto) {
        if(StringUtils.isNotBlank(dto.getName())){
            long count =performanceAppraisalService.count(Wrappers.lambdaQuery(PerformanceAppraisal.class)
                    .eq(PerformanceAppraisal::getName,dto.getName())
                    .ne(Objects.nonNull(dto.getId()),PerformanceAppraisal::getId,dto.getId()));
            VUtils.isTrue(count > 0).throwMessage("名称已经存在");
        }
    }

    private void saveProjects(PerformanceAppraisalCreateDTO createDTO,Long id,UserInfoDetails user) {
        if(CollUtil.isEmpty(createDTO.getProjects())){
            return;
        }

        List<AppraisalProjectDTO> projects = createDTO.getProjects();

        for(AppraisalProjectDTO appraisalProject : projects){
            Project newProject = projectService.getNewProject(appraisalProject.getId());
            VUtils.isTrue(Objects.isNull(newProject)).throwMessage("项目不存在 " + appraisalProject.getId());
            PerformanceAppraisalProject pap = new PerformanceAppraisalProject();
            pap.setAppraisalId(id);
            pap.setProjectId(appraisalProject.getId());
            pap.setProjectCode(newProject.getProjectCode());
            pap.setIsReAppraisal(appraisalProject.getIsReAppraisal());
            pap.setCreateBy(user.getUsername());
            pap.setCreateOn(LocalDateTime.now());
            pap.setUpdateBy(user.getUsername());
            pap.setUpdateOn(LocalDateTime.now());
            performanceAppraisalProjectService.save(pap);
        }
    }

    private void saveApplications(PerformanceAppraisalCreateDTO createDTO,Long id,UserInfoDetails user) {
        if(CollUtil.isEmpty(createDTO.getApplicationIds())){
            return;
        }

        List<Long> applicationIds = createDTO.getApplicationIds();

        for(Long applicationId : applicationIds){
            ProjectApplication projectApplication = applicationService.getById(applicationId);
            VUtils.isTrue(Objects.isNull(projectApplication)).throwMessage("应用不存在 " + applicationId);
            PerformanceAppraisalApplication paa = new PerformanceAppraisalApplication();
            paa.setAppraisalId(id);
            paa.setApplicationId(applicationId);
            paa.setCreateBy(user.getUsername());
            paa.setCreateOn(LocalDateTime.now());
            paa.setUpdateBy(user.getUsername());
            paa.setUpdateOn(LocalDateTime.now());
            performanceAppraisalApplicationService.save(paa);
        }
    }

    private void saveProjects(PerformanceAppraisalEditDTO editDTO,UserInfoDetails user) {
        if(Objects.isNull(editDTO.getProjects())){
            return;
        }

        //删除先
        performanceAppraisalProjectService.remove(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId,editDTO.getId()));
        List<AppraisalProjectDTO> projects = editDTO.getProjects();
        for(AppraisalProjectDTO appraisalProject : projects){
            Project newProject = projectService.getNewProject(appraisalProject.getId());
            VUtils.isTrue(Objects.isNull(newProject)).throwMessage("项目不存在 " + appraisalProject.getId());
            PerformanceAppraisalProject pap = new PerformanceAppraisalProject();
            pap.setAppraisalId(editDTO.getId());
            pap.setProjectId(appraisalProject.getId());
            pap.setProjectCode(newProject.getProjectCode());
            pap.setIsReAppraisal(appraisalProject.getIsReAppraisal());
            pap.setCreateBy(user.getUsername());
            pap.setCreateOn(LocalDateTime.now());
            pap.setUpdateBy(user.getUsername());
            pap.setUpdateOn(LocalDateTime.now());
            performanceAppraisalProjectService.save(pap);
        }
    }

    private void saveApplications(PerformanceAppraisalEditDTO editDTO,UserInfoDetails user) {
        if(CollUtil.isEmpty(editDTO.getApplicationIds())){
            return;
        }

        //删除先
        performanceAppraisalApplicationService.remove(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId,editDTO.getId()));
        List<Long> applicationIds = editDTO.getApplicationIds();

        for(Long applicationId : applicationIds){
            ProjectApplication projectApplication = applicationService.getById(applicationId);
            VUtils.isTrue(Objects.isNull(projectApplication)).throwMessage("应用不存在 " + applicationId);
            PerformanceAppraisalApplication paa = new PerformanceAppraisalApplication();
            paa.setAppraisalId(editDTO.getId());
            paa.setApplicationId(applicationId);
            paa.setCreateBy(user.getUsername());
            paa.setCreateOn(LocalDateTime.now());
            paa.setUpdateBy(user.getUsername());
            paa.setUpdateOn(LocalDateTime.now());
            performanceAppraisalApplicationService.save(paa);
        }
    }
}
